import socket
import time
from typing import Any, Generator

import pytest

from pendo import Pendo
from pendo.types import PendoTrackEvent


@pytest.fixture(scope="function")
def pendo_client() -> Generator:
    pendo = Pendo(pendo_integration_key="test", extra_headers={})
    yield pendo


@pytest.fixture(scope="function")
def pendo_client_extra_headers() -> Generator:
    pendo = Pendo(pendo_integration_key="test", extra_headers={"x-test-header": "test"})
    yield pendo


@pytest.fixture()
def pendo_track_event() -> PendoTrackEvent:
    return PendoTrackEvent(
        event="TestEvent",
        visitorId="TestVisitorId",
        accountId="TestAccountId",
        timestamp=int(time.time() * 1000),
    )


@pytest.fixture()
def pendo_track_event_with_props_and_context() -> PendoTrackEvent:
    return PendoTrackEvent(
        event="TestEvent",
        visitorId="TestVisitorId",
        accountId="TestAccountId",
        timestamp=int(time.time() * 1000),
        properties={"test": "test"},
        context={"test": "test"},
    )


def guard(*args: Any, **kwargs: Any) -> None:
    raise Exception(
        "No external network access allowed during tests."
    )  # pragma: no cover


socket.socket = guard  # type: ignore



from enum import Enum, unique
from typing import Dict, Union

from pydantic import BaseModel, StrictInt, StrictStr


@unique
class PendoEventType(str, Enum):
    track = "track"


class PendoProperties(BaseModel):
    ...


class PendoContext(BaseModel):
    ...


class PendoTrackEvent(BaseModel):
    event: StrictStr
    visitorId: StrictStr
    accountId: StrictStr
    timestamp: StrictInt
    type: PendoEventType = PendoEventType.track
    properties: Union[Dict, PendoProperties] = dict()
    context: Union[Dict, PendoContext] = dict()


class Response(BaseModel):
    status_code: StrictInt


class PendoResponse(Response):
    pass



from typing import Dict, Optional, Union

import requests

from pendo.const import PENDO_URL
from pendo.types import PendoResponse, PendoTrackEvent

from .exc import PendoException


class Pendo:
    """Pendo.

    An unofficial Python package for server-side Pendo integrations.

    Attributes:
        pendo_integration_key (str): The Pendo integration key. A Pendo Admin can
            access this key in your app settings
            (Subscription Settings > Choose your App > App Details).
        extra_headers (Optional[Dict]): Extra headers to be sent with the request.
    """

    def __init__(
        self, pendo_integration_key: str, extra_headers: Optional[Dict] = dict()
    ) -> None:
        self.pendo_integration_key = pendo_integration_key
        self.extra_headers = extra_headers

    def __headers(self) -> Dict:
        """Private method to return the headers for the request.

        Returns:
            Dict: The private headers.
        """
        return {
            "Content-Type": "application/json; charset=utf-8",
            "x-pendo-integration-key": self.pendo_integration_key,
        }

    def headers(self) -> Dict:
        """Return the headers for the request.

        Returns:
            Dict: The headers.
        """
        if self.extra_headers:
            return {**self.extra_headers, **self.__headers()}
        return self.__headers()

    def track(
        self, event: Union[Dict, PendoTrackEvent], url: Optional[str] = None
    ) -> PendoResponse:
        """Track an event.

        Args:
            event (Union[Dict, PendoTrackEvent]): The track event to be used as a POST
                request body.
            url (Optional[str]): The URL to send the POST request to. Defaults to the
                PENDO_URL constant. Defaults as None.

        Returns:
            PendoResponse: The response.
        """
        if url is None:
            url = PENDO_URL
        if isinstance(event, dict):
            event = PendoTrackEvent(**event)
        response = requests.post(
            f"{url}/data/track", json=event.dict(), headers=self.headers()
        )
        if not response.ok:
            raise PendoException(
                f"Request exited with status {response.status_code}: {response.reason}"
            )
        return PendoResponse(status_code=response.status_code)



import time
from unittest import mock

import pytest

from pendo import Pendo
from pendo.exc import PendoException
from pendo.types import PendoTrackEvent


@mock.patch(
    "pendo.main.requests.post",
    return_value=mock.Mock(
        status_code=200, json=mock.Mock(return_value={"status": "ok"}), ok=True
    ),
)
def test_pendo(
    mock_post: mock.MagicMock, pendo_client: Pendo, pendo_track_event: PendoTrackEvent
) -> None:
    response = pendo_client.track(event=pendo_track_event)
    assert response.status_code == 200


@mock.patch(
    "pendo.main.requests.post",
    return_value=mock.Mock(
        status_code=500,
        reason="Internal Server Error",
        ok=False,
    ),
)
def test_pendo_catch_exception(
    mock_post: mock.MagicMock, pendo_client: Pendo, pendo_track_event: PendoTrackEvent
) -> None:
    with pytest.raises(PendoException) as exc_info:
        pendo_client.track(event=pendo_track_event)
    assert (
        exc_info.value.args[0]
        == "Request exited with status 500: Internal Server Error"
    )


@mock.patch(
    "pendo.main.requests.post",
    return_value=mock.Mock(
        status_code=200, json=mock.Mock(return_value={"status": "ok"}), ok=True
    ),
)
def test_pendo_with_extra_headers(
    mock_post: mock.MagicMock,
    pendo_client_extra_headers: Pendo,
    pendo_track_event: PendoTrackEvent,
) -> None:
    response = pendo_client_extra_headers.track(event=pendo_track_event)
    assert response.status_code == 200


@mock.patch(
    "pendo.main.requests.post",
    return_value=mock.Mock(
        status_code=200, json=mock.Mock(return_value={"status": "ok"}), ok=True
    ),
)
def test_pendo_track_event_with_props_and_context(
    mock_post: mock.MagicMock,
    pendo_client: Pendo,
    pendo_track_event_with_props_and_context: PendoTrackEvent,
) -> None:
    response = pendo_client.track(event=pendo_track_event_with_props_and_context)
    assert response.status_code == 200


@mock.patch(
    "pendo.main.requests.post",
    return_value=mock.Mock(
        status_code=200, json=mock.Mock(return_value={"status": "ok"}), ok=True
    ),
)
def test_pendo_track_event_dict_input(
    mock_post: mock.MagicMock,
    pendo_client: Pendo,
) -> None:
    response = pendo_client.track(
        {
            "event": "TestEvent",
            "visitorId": "TestVisitorId",
            "accountId": "TestAccountId",
            "timestamp": int(time.time() * 1000),
        }
    )
    assert response.status_code == 200


@mock.patch(
    "pendo.main.requests.post",
    return_value=mock.Mock(
        status_code=200, json=mock.Mock(return_value={"status": "ok"}), ok=True
    ),
)
def test_pendo_track_event_custom_event(
    mock_post: mock.MagicMock,
    pendo_client: Pendo,
) -> None:
    response = pendo_client.track(
        {
            "event": "TestEvent",
            "visitorId": "TestVisitorId",
            "accountId": "TestAccountId",
            "timestamp": int(time.time() * 1000),
            "properties": {"test": "test", "more": "more"},
            "context": {"more": "more", "testing": "testing", "one": {"two": "three"}},
        }
    )
    assert response.status_code == 200
